function getMethodSpec(deviceId, instanceId, ccId, methodclass, method) {
	var _method_spec = null;

	if (ZWaveAPIData.devices[deviceId].instances[instanceId].commandClasses[ccId][methodclass]) {
		// method class is defined. getting the method
		_method_spec = ZWaveAPIData.devices[deviceId].instances[instanceId].commandClasses[ccId][methodclass][method];
	} else {
		// device, instance and CC exist and method class is not provided
		// server has not provided us the method, since it is not capable - we will generate it automatically
		switch (ccId) {
			case 0x20:
				switch (method) {
					case 'Get':
						_method_spec = [];
						break;
					
					case 'Set':
						_method_spec = [{"label":"Level","type":{"enumof":[{"label":"Off","type":{"fix":{"value":0}}},{"label":"Dimmer level","type":{"range":{"max":99,"min":0}}},{"label":"Full","type":{"fix":{"value":99}}},{"label":"On","type":{"fix":{"value":255}}}]}}];
						break;
				}
				break;
			
			case 0x25:
				switch (method) {
					case 'Get':
						_method_spec = [];
						break;
					
					case 'Set':
						_method_spec = [{"label":"Level","type":{"enumof":[{"label":"Off","type":{"fix":{"value":0}}},{"label":"On","type":{"fix":{"value":255}}}]}}];
						break;
				}
				break;
			
			case 0x26:
				switch (method) {
					case 'Get':
						_method_spec = [];
						break;
					
					case 'Set':
						_method_spec = [{"label":"Level","type":{"enumof":[{"label":"Off","type":{"fix":{"value":0}}},{"label":"Dimmer level","type":{"range":{"max":99,"min":0}}},{"label":"Full","type":{"fix":{"value":99}}},{"label":"On","type":{"fix":{"value":255}}}]}}];
						break;
				}
				break;

			case 0x27:
				switch (method) {
					case 'Get':
						_method_spec = [];
						break;
					
					case 'Set':
						_method_spec = [{"label":"Mode","type":{"enumof":[{"label":"Not in switch all group","type":{"fix":{"value":0}}},{"label":"In switch all off group only","type":{"fix":{"value":1}}},{"label":"In switch all on group only","type":{"fix":{"value":2}}},{"label":"In switch all on and off groups","type":{"fix":{"value":255}}}]}}];
						break;
				}
				break;
		
			case 0x75:
				switch (method) {
					case 'Get':
						_method_spec = [];
						break;
					
					case 'Set':
						_method_spec = [{"label":"Local operations","type":{"enumof":[{"label":"Unprotected","type":{"fix":{"value":0}}},{"label":"Protection by sequence","type":{"fix":{"value":1}}},{"label":"No operation possible","type":{"fix":{"value":2}}}]}}]
						break;
				}
				break;

			case 0x84:
				switch (method) {
					case 'Set':
						_method_spec = [{"label":"Wakeup time, seconds","type":{"range":{"max":16777215,"min":0}}},{"label":"to Node","type":{"node":{}}}];
						break;
				}
				break;
		
			case 0x62:
				switch (method) {
					case 'Set':
						_method_spec = [{"label":"State","type":{"enumof":[{"label":"Open","type":{"fix":{"value":0x00}}},{"label":"Close","type":{"fix":{"value":0xff}}},{"label":"Open from inside","type":{"fix":{"value":0x10}}},{"label":"Open from outside","type":{"fix":{"value":0x20}}}]}}];
						break;
				}
				break;
		
		}
	};

	return _method_spec;
}
